import React, { useRef, useState, useContext, forwardRef, useImperativeHandle, useMemo, useLayoutEffect } from 'react'
import { ConfigProvider, message, Modal, Button } from 'antd'
import { ExclamationCircleOutlined } from '@ant-design/icons'
import Generator, { defaultSettings, defaultCommonSettings, defaultGlobalSettings } from 'fr-generator'
import { settings, baseSettings, globalSettings } from './config'
import widgets from '../widgets'
import { saveTableConfig, GetTableConfigJson, ReloadTableFields } from '../../apis/process'
import FormRender from '../FormRender'
import { isObject } from '../../utils'
import Drag from '../components/Drag'
import styles from '../../main.less'

const FormDesigner = (props, ref) => {

  useImperativeHandle(ref, () => (
    {
      clear,
      preview,
      submit,
    }
  ))

  const { tableName, extra } = props

  const { getPrefixCls } = useContext(ConfigProvider.ConfigContext)
  const prefixCls = getPrefixCls('pandaXform')
  const prefixClsPandaXform = getPrefixCls()
  const [visible, setVisible] = useState(false)
  const [schema, setSchema] = useState({})
  const [fieldName, setFieldName] = useState([])
  const designerRef = useRef(null)
  const formRenderRef = useRef(null)

  const settingsParent = useMemo(() => {
    let settingsParent = []
    settings.forEach(v => {
      v.widgets.forEach(s => {
        s.schema.tableNameParent = tableName
      })
      settingsParent.push(v)
    })
    return settingsParent
  }, [settings, tableName])

  const getJSON = (json, fieldName) => {
    let properties = json?.properties
    let parent = {}
    let paths = []
    let title = '(未分组)'
    let ungroupedChild = {}
    if (isObject(properties)) {
      for (let v in properties) {
        if (properties[v].type === 'object') {
          parent[v] = properties[v]
          if (v !== '(未分组)') {
            paths.push(v)
          } else {
            title = properties[v].title
          }
          let child = properties[v]?.properties
          if (isObject(child)) {
            for (let s in child) {
              let IsSystemField = false
              fieldName.forEach(j => {
                if (j.name === s) {
                  IsSystemField = j.IsSystemField
                }
              })
              child[s].tableNameParent = tableName
              child[s].tableTypeParent = json.tableType
              child[s].IsSystemField = IsSystemField
              if (v === '(未分组)') {
                ungroupedChild[s] = child[s]
              }
            }
          }
        } else {
          ungroupedChild[v] = {
            ...properties[v],
            tableNameParent: tableName
          }
        }
      }
    }
    if (JSON.stringify(ungroupedChild) !== '{}') {
      parent['(未分组)'] = {
        title: title,
        type: "object",
        collapsed: false,
        groupBase: {},
        properties: {
          ...ungroupedChild
        }
      }
      paths.push('(未分组)')
    }
    return {
      ...json,
      paths,
      properties: parent,
    }
  }

  const getTableConfig = async (tableName) => {
    const { code, data, msg } = await GetTableConfigJson({ tableName })
    const res = await ReloadTableFields({ tableName })
    if (code === 0 && res.data && res.data.root) {
      if (data && typeof data === 'string' && Array.isArray(res.data.root)) {
        setFieldName(res.data.root)
        let json = getJSON(JSON.parse(data), res.data.root)
        designerRef?.current?.setValue(json)
        setSchema(json)
      }
    } else {
      message.error(msg)
    }
  }

  useLayoutEffect(() => {
    if (tableName) {
      getTableConfig(tableName)
    }
  }, [tableName])

  const clear = () => {
    Modal.confirm({
      title: '确定要清空该表单配置吗?',
      icon: <ExclamationCircleOutlined />,
      centered: true,
      cancelText: '取消',
      okText: '确定',
      onCancel: () => {
        console.log('取消清空')
      },
      onOk: () => {
        const schema = designerRef.current.getValue()
        let newschema = {}
        for (let key in schema) {
          if (key !== 'properties') {
            newschema[key] = schema[key]
          }
        }
        designerRef.current.setValue(newschema)
      }
    })
  }

  const preview = () => {
    const errors = designerRef.current.getErrorFields()
    if (errors.length) {
      return message.error('请按照提示完善表单内容')
    }
    setVisible(true)
    let json = getJSON(designerRef.current.getValue(), fieldName)
    setSchema(json)
    designerRef.current.setValue(json)
  }

  const submit = async () => {
    const errors = designerRef.current.getErrorFields()
    if (errors.length) {
      return message.error('请按照提示完善表单内容')
    }
    let json = getJSON(designerRef.current.getValue(), fieldName)
    let groups = Object.keys(json.properties)
    if (groups.some(v => v.includes('_'))) {
      return message.error('分组包含重复字段!')
    }
    designerRef.current.setValue(json)
    const { code, data, msg } = await saveTableConfig(json)
    if (code === 0) {
      message.info('保存成功')
    } else {
      message.error(msg)
    }
  }

  const extraButtons = useMemo(() => {
    if (extra) {
      return [
        false,
        false,
        true,
        true,
        {
          text: '清空',
          onClick: clear
        },
        {
          text: '预览',
          onClick: preview
        },
        {
          text: '提交',
          type: 'primary',
          onClick: submit
        }
      ]
    }
    return [false, false, false, false]
  }, [extra, fieldName])

  const getFields = (schema) => {
    let array = []
    let parent = schema?.properties
    if (isObject(parent)) {
      for (let v in parent) {
        let child = parent[v]?.properties
        if (isObject(child)) {
          for (let s in child) {
            array.push({ ...child[s], fieldName: s })
          }
        }
      }
    }
    return array
  }

  const onSchemaChange = (schema) => {
    let fields = getFields(schema)
    window.designer = fields
  }

  const testSubmit = async () => {
    const { formValue, relationForm, errors } = await formRenderRef?.current?.getValues()
    if (errors.length) {
      return message.error('表单校验未通过!')
    }
  }

  const canDelete = (values) => {
    const { $id, tableTypeParent, IsSystemField, title, type } = values
    if (tableTypeParent === '物联设备表' && IsSystemField) {
      if (fieldName.some(v => v.name === $id)) {
        message.info(`表类型为:【物联设备表】内部字段【${$id}】不允许删除!`)
        return false
      }
    }
    if (type === 'object' && ($id === '#/物联数据' || $id === '物联数据')) {
      message.info(`表类型为:【物联设备表】包含内部字段不允许删除`)
      return false
    }
    return true
  }

  if (!tableName) {
    return null
  }

  return (
    <div className={styles.pandaXform} style={{ width: '100%', height: '100%', background: 'white' }}>
      <div style={{ height: '100%' }}>
        <Generator
          // configProvider={{ prefixCls: prefixClsPandaXform }}
          canDelete={canDelete}
          mapping={{
            object: 'Header',
          }}
          validation={true}
          ref={designerRef}
          extraButtons={extraButtons}
          widgets={widgets}
          settings={settingsParent}
          commonSettings={baseSettings}
          globalSettings={globalSettings}
          onSchemaChange={onSchemaChange}
        />
        <Drag
          title={tableName}
          width='80%'
          bodyStyle={{ height: '700px', overflow: 'auto' }}
          onOk={() => setVisible(false)}
          onCancel={() => setVisible(false)}
          visible={visible}
          getContainer={false}
          destroyOnClose
          footer={
            <div>
              <Button onClick={() => setVisible(false)}>取消</Button>
              <Button onClick={testSubmit}>测试提交</Button>
            </div>
          }
        >
          <FormRender
            ref={formRenderRef}
            schemaValues={{ formJson: schema }}
          />
        </Drag>
      </div>
    </div>
  )

}

export default forwardRef(FormDesigner)